#----------------------------------------------------------
#  		Autocorrelation
#----------------------------------------------------------
#  * Function :
#   	10th order autocorrelation routine
#  * Assumption and Constraint :
#
#  * Input Values :
#	RAM0[10~1a] - Input Buffer A
#	RAM1[10~1a] - Input Buffer B
#
#  * Output Values :
#	RAM1[20~2a] - Autocorrelation Result
#
#  * Temporal Values :
#	RAM0[00] - Input table
#	RAM1[01] - Counter l
#  * Computation time :
#       10-tap length : maximum 152 cycles
#-----------------------------------------------------------
	org	0000
	dw	0004
	dw	0004
	dw	0004
	dw	0004
	org	0004
    ## Autocorrelation routine
    autocorrelation():
    # initialize
    	ld	a, st
    	ori 	a, 0004
    	ld	st, a
    	mod	f, setop
        ldi	r5, 20
    	ldi	(r3|01), 00	# init value of l = 0xffff
        ldi	(r3|00), @input_table
        ldi	r0, 10
        ldi	r4, 10
        ldi	a, 10		# N = 10
    input_load_loop:
	subi	1
	bra	n=1, @autocorr_start
        ld	x, ((r3|00))		# input N sample
	ld	(r0+), x
	ld	(r4+), x
        bra	always, @input_load_loop
        
    autocorr_start:
        ldi	(r3|01), 00
    # RxN(l), l=0~L (L<=P)
    autocorr_sub_loop:
        ldi	a, 10
        ld      r0, a                   # input buffer pointer
        add	a, (r3|01)
        ld      r4, a                   # index of x(n-l)
	ldi	stack, @filtering_ret	# return addr. of filter
    	ldi	a, @autocorr_filtering
    	add	a, (r3|01)
    	ld	x, a
    	ld	a, (r3|01)
    	addi	1
    	cmpi	a, 0b		# 0a : the size of autocorr.
    	bra	n=0, @autocorr_end
    	ld	(r3|01), a
    	ldi	y, 00
    	ld	a, y
    	ld	pc, x
    filtering_ret:
    	ld	(r5+), a
    	bra	always, @autocorr_sub_loop
    autocorr_end:
    	bra	always, @autocorr_end
    # autocorrelation filtering
    autocorr_filtering:
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	mpya	(r4+), (r0+)
    	ret

    input_table:
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000

