#---------------------------------------------------------------#
#	Schur Algorithm                                         #
#---------------------------------------------------------------#
#  * Functiion :                                                #
#	Schur Algorithm is to get reflection coefficients       #
#	with the input of Autocorelation function [ r(0)~r(p) ] #
#                                                               #
#  * Assumption and Constration					#
#  * Input Values :                                             #
#	RAM0[10~1f] - autocorelation function R(0)~R(9)		#
#  * Temporary Values :                                         #
#       RAM0[10~1F] - A(0)~A(p)                                 #
#	RAM0[20~2F] - B(0)~B(p)                                 #
#  * Output Values :                                            #
#	RAM1[20~2F] - K(0)~K(p)                                 #
#  * Number of cycles                                           #
#	maximum 3591 cycles					#
#---------------------------------------------------------------#
	org	0000
	dw      0004
	dw	0004
	dw	0004
	dw	0004
	org	0004

    # load autocorrelation values R(0)~R(10)
    	mod	f, setop
    	ld	a, st
    	ori	a, 0000
    	ld	st, a
    	andi	00
    	ldi	(r3|01), @autocorr_table
    	ldi	r0, 10
    	ldi	r2, 21
    	ldi	a, 0b
    autocorr_load_loop:
    	ld	x, ((r3|01))
    	ld	(r0+), x
    	ld	(r2+), x
    	subi	1
    	bra	z=0, @autocorr_load_loop
    Schur():
    # assign pointers
    	ldi	r0, 10			# A pointer
    	ldi	r4, 21			# K pointer
        ldi	r2, 22			# B pointer
    # get K1
    	ld	a, (r0+)                # load R(0), r0=010
    	ld	(r7|00), a		# devisor
    	ld	a, (r0+)                # load R(1), r0=011
    	ld	(r3|01), a		# devidend high 16 bit
    	call	always, @div_r_dp()
    	ld	a, (r7|00)		# get devision output
    	mod	always, neg	 	# K(m) = - P(m)/Q(m)
    	ld      (r4), a			# save K1
    # initial for K2~K9
        ldi	(r3|10), 09		# counter = 9
    	ldi	r1, 12
    ## get K2
    # update P2(z), Q2(z)
    schur_loop:
        ld	a, r1
        ld	r0, a
        addi	10
        ld 	r2, a			# B pointer
        ld	a, (r3|10)
        ld 	(r3|11), a
        ldi	a, 00
        ld 	(r7|00), a
        ld	(r7|01), a
        ld 	(r7|10), a
        ld 	(r7|11), a
        call    always, @get_P_Q_loop
        ld      a, (r7|00)
        ld	01a, a
        ld	a, (r7|11)
        ld	029, a
        ld	a, (r7|10)
        ld	02a, a
     # calculate Km=-P(m)/Q(m)
        ld      a, (r1)
        ld	(r3|01), a		# dividend high 16 bit
        ld	a, r1
        addi	10
        ld	r1, a			# jump to B pointer
        ld	a, (r1+)
        ld	(r7|00), a		# divisor
        ld	a, r1
        subi	10
        ld	r1, a			# jump to A pointer
        call	always, @div_r_dp()
        ld	a, (r7|00)
    	mod	always, neg	 	# Km = - P(m)/Q(m)
        ld	(r4), a			# save Km
        
        ld	a, (r3|10)
        subi	1
        ld	(r3|10), a
        bra	z=0, @schur_loop
    # schur algorithm end
    schur_end:
    	bra	always, @schur_end
    	ret
                                
    get_P_Q_loop:
    # calculate P(m)
        
        mld	(r4), (r2-)		# Km-1 * B(m)
	ld	a, (r0-)		# A(m) + Km-1*B(m)
	mpya	(r4), (r0)		# dummy for addition
        ld      stack, (r7|00)          # load A(m-1)
	ld	(r7|00), a		# save updated A(m)
    
    # calculate Q(m)
        mld	(r4), (r0)		# Km-1 * A(m-1)
    	ld	a, (r2)			# B(m-1) + Km-1*A(m-1)
	ld	(r0+), stack		# save A(m-1)
	mpya 	(r4), (r0+)		# dummy for addition
	ld	stack, (r7|10)
	ld	(r2+), stack		# save updated B(m-1)
	ld	(r7|10), a		# save updated B(m)
	mld	(r4), (r2+)		# increase B pointer to B(m+1)
	
	ld	a, (r3|11)		# counter
	subi	1
	ld	(r3|11), a
	bra	z=0, @get_P_Q_loop
	ld	-, (r4+)		# increase K pointer
	ret


#-------------------------------------------------------#
#   mult_r_dp.sc					#
#-------------------------------------------------------#
#   * Function:                                         #
#     32bit/16bit division with the result 16bit        #
#     quotient and remainder				#
#   * Design Assumption and Constraints:		#
#     |dividend| < |divisor|				#
#   * Input values                                      #
#	   RAM0[00] = Dividend low 16 bit		#
#          RAM0[01] = Dividend high 16 bit		#
#          RAM1[00] = Divisor            		#
#   * Return values                                     #
#          RAM0[00] = Remainder_low		        #
#	   RAM0[01] = Remainder_high			#
#          RAM1[00] = Quotient 	                	#
#   * Temporal value                                    #
#          RAM0[00] = Dividend higher 16 bit buffer    	#
#          RAM0[01] = Dividend lower 16 bit buffer    	#
#	   RAM1[01] = Sign of quotient			#
#	   r0 = counter of subc loop			#
#   * Altered Registers					#
#          Acc, r3, r7, stack, st                       #
#   * Computation cycles                                #
#          maximum ?? cycles                            #
#                                                       #
#-------------------------------------------------------#
quot_sgn:	equ	101
remainder_low:	equ	000
remainder_high:	equ     001
dividend_low:	equ	000
dividend_high:	equ	001
divisor:	equ	100
quotient:	equ	100

#--------------------------------------------------------
    div_r_dp():
        ldi	r0, 0d			# subc count number
    # get sign of quotient
        mld	(r7|00), (r3|01) 	# divisor * dividend_high
        ld	a, p
        ld	@quot_sgn, a   	        # sign of quotient
        ld	a, @dividend_high
        ld	ext7, (r3|00)		# dividend_low
        mod	always, abs             # to make positive denom
        ld	@dividend_high, a
        ld      (r3|00), ext7
        ld	a, @dividend_low
        andi	a, fffe
        ld	@dividend_low, a
        ld	a, @divisor             # to make positive numera
        mod	always, abs
        ld	@divisor, a
	ldi	y, 4000         	# for divisor shift
    	ld	x, (r7|00) 		# divisor
        ld      ext7, (r3|00)           # dividend low 16bit
    sub_loop:
        ld	a, @dividend_high	# dividend higher 16bit
	cmp	a, p
        bra	n=1, @neg_result
        sub	a, p       		# make partial remainder
        mod	always, inc             # generate quotient
        ld	@dividend_high, a	# save acc
        bra 	always, @sub_loop
    neg_result:                         # if remainder<divisor
    	mod	always, shl		# double remainder
        ld	@dividend_high, a	# save acc
        ld	a, r0
        subi	1
        ld	r0, a
        bra	n=1, @div_r_dp_end
    	bra	always, @sub_loop
    div_r_dp_end:
    	ld	(r7|00), ext7		# save quotient
    	andi	a, 00
    	ld	x, (r3|01)		# remainder
    	ldi	y, 0002
    	add	a, p
    	ld	@remainder_high, a
    	ld	@remainder_low, ext7
    	ld	a, @quot_sgn		# check quotient sign
    	and	a, -
    	ld	a, (r7|00)      	# quotent
    	bra	n=0, @div_r_dp_ret
    	mod	always, neg             # apply sign
    div_r_dp_ret:
        ld	@quotient, a		# save final quotent
        ld	pc, stack

autocorr_table:
	dw	7fff
	dw	7fbe
	dw	7eff
	dw	7dc5
	dw	7ffb
	dw	79ef
	dw	775b
	dw	745f
	dw	70fe
	dw	6d3f
	dw	6927
	
